<?php namespace Myriad\Carmen;

use Illuminate\Support\ServiceProvider;

class CarmenServiceProvider extends ServiceProvider {

  /**
   * Indicates if loading of the provider is deferred.
   *
   * @var bool
   */
  protected $defer = true;

  /**
   * Perform post-registration booting of services.
   *
   * @return void
   */
  public function boot() {
    $this->publishes([__DIR__ . '/../config/carmen.php' => config_path('carmen.php')]);
  }

  /**
   * Register the service provider.
   *
   * @return void
   */
  public function register() {
    $this->mergeConfigFrom(__DIR__ . '/../config/carmen.php', 'carmen');

    $this->registerSquire();

    $this->app->alias('carmen', 'Myriad\Carmen\Contracts\Carmen');
    $this->app->alias('carmen', 'Myriad\Carmen\CarmenClient');
  }

  /**
   * Register the Carmen instance
   */
  private function registerSquire() {
    $this->app->singleton('carmen', function ($app) {
      $guzzle = new \GuzzleHttp\Client();
      $config = new CarmenConfiguration(config('carmen'), $guzzle);

      return new CarmenClient($config, $guzzle);
    });
  }

  /**
   * Get the services provided by the provider.
   *
   * @return array
   */
  public function provides() {
    return ['carmen', 'Myriad\Carmen\Contracts\Carmen', 'Myriad\Carmen\CarmenClient'];
  }

}